@@ -14,7 +14,7 @@
 #' @param display takes the display that is suppose to be used, defaults to
 #'   the xy display
 #' @param aps target angular velocity (in radians per second)
-#' @param fps target frames per second (defaults to 30)
+#' @param fps target frames per second (defaults to 15, to accommodate RStudio graphics device)
 #' @param max_frames the maximum number of bases to generate.  Defaults to
 #'   Inf for interactive use (must use Ctrl + C to terminate), and 1 for
 #'   non-interactive use.
@@ -25,7 +25,7 @@
 #' @references Hadley Wickham, Dianne Cook, Heike Hofmann, Andreas Buja
 #'   (2011). tourr: An R Package for Exploring Multivariate Data with
 #'   Projections. Journal of Statistical Software, 40(2), 1-18.
-#'   \url{http://www.jstatsoft.org/v40/i02/}.
+#'   \url{https://www.jstatsoft.org/v40/i02/}.
 #' @export
 #' @examples
 #' f <- flea[, 1:6]
@@ -33,11 +33,29 @@
 #' # or in short
 #' animate(f)
 #' animate(f, max_frames = 30)
-#'
-#' \dontrun{animate(f, max_frames = 10, fps = 1, aps = 0.1)}
-animate <- function(data, tour_path = grand_tour(), display = display_xy(), start = NULL, aps = 1, fps = 30, max_frames = Inf, rescale = TRUE, sphere = FALSE, ...) {
+#' \dontrun{
+#' animate(f, max_frames = 10, fps = 1, aps = 0.1)
+#' }
+animate <- function(data, tour_path = grand_tour(), display = display_xy(),
+                    start = NULL, aps = 1, fps = 10, max_frames = Inf,
+                    rescale = TRUE, sphere = FALSE, ...) {
+  record <-
+    dplyr::tibble(
+      basis = list(),
+      index_val = numeric(),
+      info = character(),
+      method = character(),
+      alpha = numeric(),
+      tries = numeric(),
+      loop = numeric()
+    )
+  if (!is.matrix(data)) {
+    message("Converting input data to the required matrix format.")
+    data <- as.matrix(data)
+  }
+
   if (rescale) data <- rescale(data)
-  if (sphere) data  <- sphere_data(data)
+  if (sphere) data <- sphere_data(data)
 
   # By default, only take single step if not interactive
   # This is useful for the automated tests run by R CMD check
@@ -53,8 +71,8 @@ animate <- function(data, tour_path = grand_tour(), display = display_xy(), star
     fps <- 19
   }
 
-  tour <- new_tour(data, tour_path, start)
-  start <- tour(0)
+  tour <- new_tour(data, tour_path, start, ...)
+  start <- tour(0, ...)
   bs <- 1
   bases <- array(NA, c(ncol(data), ncol(start$target), bs))
 
@@ -66,38 +84,50 @@ animate <- function(data, tour_path = grand_tour(), display = display_xy(), star
   b <- 0
   i <- 0
 
-  tryCatch({
-    while(i < max_frames) {
-      i <- i + 1
-      step <- tour(aps / fps)
-      if (step$step == 1) {
-        b <- b + 1
-        if (b > bs) {
-          bases <- c(bases, rep(NA, bs * dim(bases)[1] * dim(bases)[2]))
-          dim(bases) <- c(ncol(data), ncol(start$target), 2 * bs)
-          bs <- 2 * bs
+  tryCatch(
+    {
+      while (i < max_frames) {
+        i <- i + 1
+        step <- tour(aps / fps, ...)
+        if (step$step == 1) {
+          b <- b + 1
+          if (b > bs) {
+            bases <- c(bases, rep(NA, bs * dim(bases)[1] * dim(bases)[2]))
+            dim(bases) <- c(ncol(data), ncol(start$target), 2 * bs)
+            bs <- 2 * bs
+          }
+          bases[, , b] <- step$target
         }
-        bases[, , b] <- step$target
-      }
 
-      dev.hold()
-      on.exit(dev.flush())
-      if (plat$os == "win" || plat$iface == "rstudio") {
-        display$render_frame()
-      } else {
-        display$render_transition()
+        dev.hold()
+        on.exit(dev.flush())
+        if (plat$os == "win" || plat$iface == "rstudio") {
+          display$render_frame()
+        } else {
+          display$render_transition()
+        }
+        display$render_data(data, step$proj, step$target)
+        dev.flush()
+        if (step$step < 0) break # break after rendering final projection
+        Sys.sleep(1 / fps)
       }
-      display$render_data(data, step$proj, step$target)
+    },
+    interrupt = function(cond) {
       dev.flush()
-      if (step$step < 0) break #break after rendering final projection
-      Sys.sleep(1 / fps)
+      return()
     }
-  }, interrupt = function(cond) {
-    dev.flush()
-    return()
-  })
+  )
 
-  invisible(bases[, , seq_len(b)])
+  if (b != 0) {
+    invisible(bases[, , seq_len(b)])
+  }
+
+  invisible(record)
+
+  # Need a better way to clean up global variables than this
+  # suppressWarnings(rm(tries, cur_index, current, t0, record, envir = globalenv()))
 }
 
 rstudio_gd <- function() identical(names(dev.cur()), "RStudioGD")
+
+# globalVariables("record")
