@@ -17,30 +17,29 @@
 #' frozen <- matrix(NA, nrow = 4, ncol = 2)
 #' frozen[3, ] <- .5
 #' animate_xy(flea[, 1:4], frozen_tour(2, frozen))
-#'
 #' \dontrun{
-#'   # Doesn't work - a bug?
-#'   frozen <- matrix(NA, nrow = 4, ncol = 2)
-#'   frozen[1, 1] <- 0.5
-#'   animate_xy(flea[, 1:4], frozen_tour(2, frozen))
+#' # Doesn't work - a bug?
+#' frozen <- matrix(NA, nrow = 4, ncol = 2)
+#' frozen[1, 1] <- 0.5
+#' animate_xy(flea[, 1:4], frozen_tour(2, frozen))
 #'
-#'   # Doesn't work - a bug?
-#'   frozen <- matrix(NA, nrow = 4, ncol = 2)
-#'   frozen[, 1] <- 1/2
-#'   animate_xy(flea[, 1:4], frozen_tour(2, frozen))
+#' # Doesn't work - a bug?
+#' frozen <- matrix(NA, nrow = 4, ncol = 2)
+#' frozen[, 1] <- 1 / 2
+#' animate_xy(flea[, 1:4], frozen_tour(2, frozen))
 #'
-#'   # Doesn't work - a bug?
-#'   frozen[3, ] <- c(0, 1)
-#'   animate_xy(flea[, 1:4], frozen_tour(2, frozen))
+#' # Doesn't work - a bug?
+#' frozen[3, ] <- c(0, 1)
+#' animate_xy(flea[, 1:4], frozen_tour(2, frozen))
 #'
-#'   # Doesn't move, which is correct - no free variables
-#'   frozen[4, ] <- .2
-#'   animate_xy(flea[, 1:4], frozen_tour(2, frozen))
+#' # Doesn't move, which is correct - no free variables
+#' frozen[4, ] <- .2
+#' animate_xy(flea[, 1:4], frozen_tour(2, frozen))
 #'
-#'   # Doesn't work - a bug?
-#'   frozen <- matrix(NA, nrow = 4, ncol = 2)
-#'   frozen[, 1] <- 1/2
-#'   animate_xy(flea[, 1:4], frozen_tour(2, frozen))
+#' # Doesn't work - a bug?
+#' frozen <- matrix(NA, nrow = 4, ncol = 2)
+#' frozen[, 1] <- 1 / 2
+#' animate_xy(flea[, 1:4], frozen_tour(2, frozen))
 #' }
 #' # Two frozen variables in five 5.
 #' frozen <- matrix(NA, nrow = 5, ncol = 2)
@@ -48,10 +47,13 @@
 #' frozen[4, ] <- c(-.2, .2)
 #' animate_xy(flea[, 1:5], frozen_tour(2, frozen))
 frozen_tour <- function(d = 2, frozen) {
-  generator <- function(current, data) {
-    if (is.null(current)) return(basis_init(ncol(data), d))
+  generator <- function(current, data, ...) {
+    if (is.null(current)) {
+      return(basis_init(ncol(data), d))
+    }
 
-    basis_random(ncol(data), d)
+    target <- basis_random(ncol(data), d)
+    list(target = target)
   }
 
   check_freezer_safe(frozen)
@@ -75,14 +77,17 @@ frozen_tour <- function(d = 2, frozen) {
 #' frozen[3, ] <- .5
 #' animate_xy(flea[, 1:4], frozen_guided_tour(frozen, holes()))
 frozen_guided_tour <- function(frozen, index_f, d = 2, max.tries = 25) {
-  generator <- function(current, data) {
-    if (is.null(current)) return(basis_init(ncol(data), d))
+  generator <- function(current, data, tries, ...) {
+    if (is.null(current)) {
+      return(basis_init(ncol(data), d))
+    }
 
     index <- function(proj) {
       index_f(as.matrix(data) %*% proj)
     }
 
-    search_frozen_geodesic(current, index, max.tries, frozen = frozen)
+    basis <- search_frozen_geodesic(current, index, tries, max.tries, frozen = frozen)
+    list(target = basis$target, index = index)
   }
 
   check_freezer_safe(frozen)
@@ -96,7 +101,7 @@ frozen_guided_tour <- function(frozen, index_f, d = 2, max.tries = 25) {
 check_freezer_safe <- function(frozen) {
   stopifnot(is.matrix(frozen))
 
-  lengths <- colSums(frozen ^ 2, na.rm = TRUE)
+  lengths <- colSums(frozen^2, na.rm = TRUE)
   if (any(lengths >= 1)) {
     stop("Columns of frozen matrix must have squared norm < 1", call. = FALSE)
   }
@@ -134,7 +139,7 @@ thaw <- function(input, frozen) {
   fixed <- !is.na(frozen)
 
   input <- normalise(input)
-  frozen_lengths <- colSums(frozen ^ 2, na.rm = TRUE)
+  frozen_lengths <- colSums(frozen^2, na.rm = TRUE)
 
   input <- sweep(input, 2, sqrt(1 - frozen_lengths), "*")
   input[fixed] <- frozen[fixed]
